@@ -239,7 +239,6 @@ gpm_stats_update_smooth_data (GPtrArray *list)
 
 	/* convert the y data to a EggArrayFloat array */
 	raw = egg_array_float_new (list->len);
-	convolved = egg_array_float_new (list->len);
 	for (i=0; i<list->len; i++) {
 		point = (GpmPointObj *) g_ptr_array_index (list, i);
 		egg_array_float_set (raw, i, point->y);
@@ -253,7 +252,7 @@ gpm_stats_update_smooth_data (GPtrArray *list)
 	convolved = egg_array_float_convolve (outliers, gaussian);
 
 	/* add the smoothed data back into a new array */
-	new = g_ptr_array_new ();
+	new = g_ptr_array_new_with_free_func ((GDestroyNotify) gpm_point_obj_free);
 	for (i=0; i<list->len; i++) {
 		point = (GpmPointObj *) g_ptr_array_index (list, i);
 		point_new = g_new0 (GpmPointObj, 1);
@@ -510,8 +509,7 @@ gpm_stats_set_graph_data (GtkWidget *widget, GPtrArray *data, gboolean use_smoot
 		if (use_points)
 			gpm_graph_widget_data_assign (GPM_GRAPH_WIDGET (widget), GPM_GRAPH_WIDGET_PLOT_POINTS, data);
 		gpm_graph_widget_data_assign (GPM_GRAPH_WIDGET (widget), GPM_GRAPH_WIDGET_PLOT_LINE, smoothed);
-		g_ptr_array_foreach (smoothed, (GFunc) gpm_point_obj_free, NULL);
-		g_ptr_array_free (smoothed, TRUE);
+		g_ptr_array_unref (smoothed);
 	}
 
 	/* show */
@@ -535,7 +533,7 @@ gpm_stats_update_info_page_history (DkpDevice *device)
 	gint32 offset = 0;
 	GTimeVal timeval;
 
-	new = g_ptr_array_new ();
+	new = g_ptr_array_new_with_free_func ((GDestroyNotify) gpm_point_obj_free);
 	if (g_strcmp0 (history_type, GPM_HISTORY_CHARGE_VALUE) == 0) {
 		g_object_set (graph_history,
 			      "type-x", GPM_GRAPH_WIDGET_TYPE_TIME,
@@ -620,10 +618,8 @@ gpm_stats_update_info_page_history (DkpDevice *device)
 	/* present data to graph */
 	gpm_stats_set_graph_data (graph_history, new, checked, points);
 
-	g_ptr_array_foreach (array, (GFunc) dkp_history_obj_free, NULL);
-	g_ptr_array_free (array, TRUE);
-	g_ptr_array_foreach (new, (GFunc) gpm_point_obj_free, NULL);
-	g_ptr_array_free (new, TRUE);
+	g_ptr_array_unref (array);
+	g_ptr_array_unref (new);
 out:
 	return;
 }
@@ -645,7 +641,7 @@ gpm_stats_update_info_page_stats (DkpDevice *device)
 	gboolean use_data = FALSE;
 	const gchar *type = NULL;
 
-	new = g_ptr_array_new ();
+	new = g_ptr_array_new_with_free_func ((GDestroyNotify) gpm_point_obj_free);
 	if (g_strcmp0 (stats_type, GPM_STATS_CHARGE_DATA_VALUE) == 0) {
 		type = "charging";
 		use_data = TRUE;
@@ -713,10 +709,8 @@ gpm_stats_update_info_page_stats (DkpDevice *device)
 	/* present data to graph */
 	gpm_stats_set_graph_data (graph_statistics, new, checked, points);
 
-	g_ptr_array_foreach (array, (GFunc) dkp_stats_obj_free, NULL);
-	g_ptr_array_free (array, TRUE);
-	g_ptr_array_foreach (new, (GFunc) gpm_point_obj_free, NULL);
-	g_ptr_array_free (new, TRUE);
+	g_ptr_array_unref (array);
+	g_ptr_array_unref (new);
 out:
 	return;
 }
@@ -1027,8 +1021,7 @@ gpm_stats_update_wakeups_data (void)
 		obj = g_ptr_array_index (array, i);
 		gpm_stats_add_wakeups_obj (obj);
 	}
-	g_ptr_array_foreach (array, (GFunc) dkp_wakeups_obj_free, NULL);
-	g_ptr_array_free (array, TRUE);
+	g_ptr_array_unref (array);
 }
 
 static void
@@ -1225,7 +1218,7 @@ gpm_stats_device_removed_cb (DkpClient *client, DkpDevice *device, gpointer user
 
 	/* search the list and remove the object path entry */
 	ret = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (list_store_devices), &iter);
-	do {
+	while (ret) {
 		gtk_tree_model_get (GTK_TREE_MODEL (list_store_devices), &iter, GPM_DEVICES_COLUMN_ID, &id, -1);
 		if (g_strcmp0 (id, object_path) == 0) {
 			gtk_list_store_remove (list_store_devices, &iter);
@@ -1233,7 +1226,7 @@ gpm_stats_device_removed_cb (DkpClient *client, DkpDevice *device, gpointer user
 		}
 		g_free (id);
 		ret = gtk_tree_model_iter_next (GTK_TREE_MODEL (list_store_devices), &iter);
-	} while (ret);
+	};
 }
 
 /**
@@ -1436,6 +1429,43 @@ gpm_stats_set_combo_simple_text (GtkWidget *combo_box)
 					NULL);
 }
 
+/**
+ * gpm_stats_highlight_device:
+ **/
+static void
+gpm_stats_highlight_device (const gchar *object_path)
+{
+	gboolean ret;
+	gchar *id = NULL;
+	gchar *path_str;
+	guint i;
+	GtkTreeIter iter;
+	GtkTreePath *path;
+	GtkWidget *widget;
+
+	/* check valid */
+	if (!g_str_has_prefix (object_path, "/"))
+		return;
+
+	/* we have to reuse the treeview data as it may be sorted */
+	ret = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (list_store_devices), &iter);
+	for (i=0; ret; i++) {
+		gtk_tree_model_get (GTK_TREE_MODEL (list_store_devices), &iter,
+				    GPM_DEVICES_COLUMN_ID, &id,
+				    -1);
+		if (g_strcmp0 (id, object_path) == 0) {
+			path_str = g_strdup_printf ("%i", i);
+			path = gtk_tree_path_new_from_string (path_str);
+			widget = GTK_WIDGET (gtk_builder_get_object (builder, "treeview_devices"));
+			gtk_tree_view_set_cursor_on_cell (GTK_TREE_VIEW (widget), path, NULL, NULL, FALSE);
+			g_free (path_str);
+			gtk_tree_path_free (path);
+		}
+		g_free (id);
+		ret = gtk_tree_model_iter_next (GTK_TREE_MODEL (list_store_devices), &iter);
+	}
+}
+
 /**
  * main:
  **/
@@ -1455,9 +1485,8 @@ main (int argc, char *argv[])
 	DkpDeviceType type;
 	guint i;
 	gint page;
-	const gchar *object_path;
 	gboolean checked;
-	gchar *last_device;
+	gchar *last_device = NULL;
 	guint retval;
 	GError *error = NULL;
 
@@ -1511,7 +1540,7 @@ main (int argc, char *argv[])
 	/* get UI */
 	builder = gtk_builder_new ();
 	retval = gtk_builder_add_from_file (builder, GPM_DATA "/gpm-statistics.ui", &error);
-	if (error != NULL) {
+	if (retval == 0) {
 		egg_warning ("failed to load ui: %s", error->message);
 		g_error_free (error);
 	}
@@ -1607,7 +1636,6 @@ main (int argc, char *argv[])
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "treeview_wakeups"));
 	gtk_tree_view_set_model (GTK_TREE_VIEW (widget),
 				 GTK_TREE_MODEL (list_store_wakeups));
-	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (widget));
 
 	/* add columns to the tree view */
 	gpm_stats_add_wakeups_columns (GTK_TREE_VIEW (widget));
@@ -1745,24 +1773,10 @@ main (int argc, char *argv[])
 	}
 
 	/* set the correct focus on the last device */
-	for (i=0; i < devices->len; i++) {
-		object_path = (const gchar *) g_ptr_array_index (devices, i);
-		if (last_device == NULL || object_path == NULL)
-			break;
-		if (g_strcmp0 (last_device, object_path) == 0) {
-			GtkTreePath *path;
-			gchar *path_str;
-			path_str = g_strdup_printf ("%i", i);
-			path = gtk_tree_path_new_from_string (path_str);
-			widget = GTK_WIDGET (gtk_builder_get_object (builder, "treeview_devices"));
-			gtk_tree_view_set_cursor_on_cell (GTK_TREE_VIEW (widget), path, NULL, NULL, FALSE);
-			g_free (path_str);
-			gtk_tree_path_free (path);
-		}
-	}
+	if (last_device != NULL)
+		gpm_stats_highlight_device (last_device);
 
-	g_ptr_array_foreach (devices, (GFunc) g_object_unref, NULL);
-	g_ptr_array_free (devices, TRUE);
+	g_ptr_array_unref (devices);
 
 	/* set axis */
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "combobox_history_type"));
